import base64
import random

from PIL import Image
from io import BytesIO
from typing import Dict, Any
from dataclasses import asdict

from nonebot.log import logger
from nonebot.plugin import PluginMetadata
from nonebot.adapters import Bot

from .config import *
from .fuctions import *

image_dirs = get_image_dirs(config.watermark_image_path)

__plugin_meta__ = PluginMetadata(
    name="图片水印",
    description="为bot发出的图片添加水印",
    usage="用来给bot发出的图片添加水印",
    config=Config,
)


@Bot.on_calling_api
async def _handle(bot: Bot, api: str, data: Dict[str, Any]):
    if api not in ["send_msg", "send_message"]:
        return
    logger.debug("handle start")
    for i in range(len(data["message"])):
        image_data = asdict(data["message"][i])
        if image_data["type"] != "image":
            continue
        file = image_data["data"]["file"]
        if image := str2img(file):
            image_size = image.size
            watermark_path = random.choice(image_dirs)
            watermark = Image.open(watermark_path)
            mix = tuple([int((k + j) / 2) for k, j in zip(watermark.size, image_size)])
            watermark_size = tuple([int(k * config.watermark_image_size) for k in mix])
            watermark_position = tuple(
                [k - j for k, j in zip(image_size, watermark_size)]
            )
            watermark.thumbnail(watermark_size)
            logger.debug(f"watermark_size:{watermark_size}\nimage_size:{image_size}")
            image = image.convert("RGBA")
            watermark = watermark.convert("RGBA")
            mask = watermark.split()[3]

            image.paste(watermark, watermark_position, mask)

            buffered = BytesIO()
            image.save(buffered, format="PNG")
            byte_data = buffered.getvalue()
            base64_str = base64.b64encode(byte_data).decode("utf-8")
            data["message"][i].data["file"] = "base64://" + base64_str

        else:
            logger.debug(file)



import io
import re
import os
import base64
import urllib.request
import PIL.Image as Image

from pathlib import Path
from typing import Union, List, Tuple


def str2img(string: str) -> Union[Image.Image, None]:
    """传入的MessageSegment中的file的字符串转换为PIL图片对象

    Args:
        string (str): _description_

    Returns:
        Union[Image.Image, None]: _description_
    """
    base64_pattern = r"base64://[a-zA-Z0-9+/]+"
    other_pattern = r"(?:file|http|https)://[a-zA-Z0-9+/]+"
    image1 = None
    if matcher := re.search(base64_pattern, string):
        string = matcher.group()
        string = string.replace("base64://", "")
        bytes = base64.b64decode(string)
        image_stream = io.BytesIO(bytes)
        image1 = Image.open(image_stream)
    elif matcher := re.search(other_pattern, string):
        string = matcher.group()
        bytes = urllib.request.urlopen(string).read()
        image1 = Image.open(bytes)

    return image1


def get_image_dirs(path: Union[str, Path]) -> List[str]:
    """获得传入目录下的所有图片(AI写的,错了不怪我())

    Args:
        path (Union[str, Path]): 查询的目录

    Returns:
        List[str]: 包含所有返回的图片目录
    """
    # 定义一个空列表，用来存储图片目录
    image_dirs = []
    # 遍历目录及其子目录
    for root, dirs, files in os.walk(path):
        # 遍历文件
        for file in files:
            # 判断文件是否是图片类型
            if file.endswith((".jpg", ".png")):
                # 获取文件的绝对路径
                image_path = os.path.join(root, file)
                # 将图片目录添加到列表中
                image_dirs.append(image_path)
    # 返回图片目录列表
    return image_dirs


def position(image: Image.Image, watermark: Image.Image, where: Tuple[int, int]):
    """TODO:不知道要不要做这个抽象"""
    pass



import os
from pathlib import Path
from pydantic import BaseModel, Extra
from typing import Union, Tuple, Iterable, Literal

from nonebot import get_driver

driver = get_driver()


class Config(BaseModel, extra=Extra.ignore):
    watermark_image_path: Path = (
        Path(os.path.dirname(__file__)) / "watermarker_image"
    )  # 水印图片存放目录,目录下的所有水印图片会被随机选取
    watermark_image_size: float = 0.07  # 水印相对图片的大小(保持水印原来的形状)
    watermark_image_exculed_plugin: Iterable[str] = []  # TODO: 增加对特定插件的排除
    # watermark_image_position: Union[
    #     Literal["左上", "右上", "左下", "右下", "居中"], Tuple[int, int] #以后再说,先 TODO 了
    # ] = "右下"


config = Config.parse_obj(driver.config.dict())



from setuptools import setup, find_packages

    
setup(
    name='nonebot-plugin-watermarker',

    version="0.1.1",
    description=(
        '为nonebot机器人发送的图片加上水印'

    ),
    long_description="""<div align="center">
  <p><img src="https://user-images.githubusercontent.com/91937041/235443858-85949be1-08d6-4d7a-b132-b1aed71ab943.png" width="560" alt="PoweredByNonebotLogo"></p>
  <a href="https://v2.nonebot.dev/store"><img src="https://ghproxy.com/https://github.com/A-kirami/nonebot-plugin-template/blob/resources/nbp_logo.png" width="180" height="180" alt="NoneBotPluginLogo"></a>
  <br>
  <p><img src="https://ghproxy.com/https://github.com/A-kirami/nonebot-plugin-template/blob/resources/NoneBotPlugin.svg" width="240" alt="NoneBotPluginText"></p>
</div>

<div align="center">

# nonebot-plugin-watermarker

_✨ 为你的bot发出的图片添加水印! ✨_

<a href="https://raw.githubusercontent.com/nonebot/nonebot2/master/LICENSE">
    <img src="https://img.shields.io/github/license/forchannot/nonebot_plugin_rename" alt="license">
</a>
<a href="https://pypi.python.org/pypi/nonebot-plugin-rename">
    <img src="https://img.shields.io/pypi/v/nonebot-plugin-rename.svg" alt="pypi">
</a>
<img src="https://img.shields.io/badge/python-3.8+-yellow.svg" alt="python">

</div>

<!-- TOC -->
  * [📖简介](#简介)
  * [💿 安装方法](#安装方法)
  * [⚙️插件配置项](#插件配置项)
  * [🎉目前已实现的功能](#目前已实现的功能)
  * [💡待实现的功能](#待实现的功能)
  * [💣已知bug](#已知bug)
  * [🔥鸣谢](#鸣谢)
  * [💦其他](#其他)
<!-- TOC -->

## 📖简介
为bot发出的所有图片都加上水印(有的时候水印很烦,但是帅气的水印能增加图片的美感,不是吗?)

## 💿安装方法
### ```nb脚手架```
<details>
<summary>使用 nb-cli 安装</summary>
在 nonebot2 项目的根目录下打开命令行, 输入以下指令即可安装

```cmd
nb plugin install nonebot_piugin_watermarker
```

</details>

### ```pip```
<details>
<summary>pip安装</summary>

命令行输入以下命令

```cmd
pip install nonebot_plugin_watermarker
```

打开 nonebot2 项目根目录下的 `pyproject.toml` 文件, 在 `[tool.nonebot]` 部分追加写入

    plugins = ["nonebot_plugin_rename"]

```
[tool.nonebot]
plugins = []
plugin_dirs = ["src/plugins"]
```
</details>

##  ⚙插件配置项

| 变量名      |       变量类型    |   变量概述       |
|------------|----------------------------------|----------------|
| ```watermark_image_path```   |```str```|水印图片存放目录,目录下的所有水印图片会被随机选取|
| ```watermark_image_size```   |```float```|水印相对图片的大小(保持水印原来的形状)|
| ```watermark_image_exculed_plugin```   |```List[str]```|不想被贴水印的插件(未完工)|

## 🎉目前已实现的功能

### 加水印捏
<details>
<summary>效果图</summary>

![6PHLH{(JH $R~J2H@8{{XBE](https://user-images.githubusercontent.com/91937041/235442049-67ac0b4c-1629-4d78-9858-8b411b7ebe7b.jpg)

太小了看不见放大看(

![G37PR_ KAWEUINI_ _B)H2E](https://user-images.githubusercontent.com/91937041/235442112-c35e08ed-64c4-4b09-93f6-5976bb70de60.jpg)

</details>

## 💡待实现的功能

1. -[ ] 对特定的插件的图片不进行贴水印操作
2. -[ ] 待补充.....

# 💣已知bug

水印小概率贴不上,应该是base64的原因,但是我一直找不到真正的问题源,因为有时候贴的上有时候贴不上(

# 🔥鸣谢

[Nonebot2](https://github.com/nonebot/nonebot2),不用说,没有Nonebot就没有这个插件

~~[我自己](https://github.com/X-Skirt-X),因为我做的PoweredByNonebot的Logo~~

# 💦其他

没有其他,想到再补
""",
    long_description_content_type="text/markdown",
    author='XU',
    author_email='Woyerpa@outlook.com',
    license='MIT License',
    packages=find_packages(),
    url='https://github.com/X-Skirt-X/nonebot-plugin-watermarker',
    install_requires=[
        "nb-cli",
        "nonebot2"
    ]
)